searchNodes=[{"doc":"Generates definitions to assist working with tagged value tuples, such as the ubiquitous {:ok, value} and {:error, reason}.Examplesdefmodule Status use Tagged deftagged ok(value :: term()) deftagged error(reason :: term()) endConstruct and Destructureiex&gt; require Status iex&gt; Status.ok(:computer) {:ok, :computer} iex&gt; with Status.error(reason) &lt;- {:ok, :computer}, do: raise reason {:ok, :computer}See Tagged.Constructor for further details.Type definitions_iex&gt; t Status.error @type error() :: {:error, reason :: term()} Tagged value tuple, containing reason :: term()See Tagged.Typedef for further details.Pipe selective executioniex&gt; require Status iex&gt; import Status, only: [ok: 1, with_ok: 2] iex&gt; ok(:computer) |&gt; with_ok(&amp; &quot;OK, \#{&amp;1}&quot;) &quot;OK, computer&quot;See Tagged.PipeWith for further details.Sum Algebraic Data Type: Binary TreeA module that defines some tagged values, a composit type, and guard of those, forms a Sum Algebraic Data Type, also known as a Tagged Union.defmodule BinTree do use Tagged deftagged tree(left :: t(), right :: t()) deftagged leaf(value :: term()) deftagged nil, as: empty() @type t() :: tree() | leaf() | empty() defguard is_t(x) when is_tree(x) or is_leaf(x) or is_empty(x) @spec leaves(tree()) :: [term()] def leaves(empty()), do: [] def leaves(leaf(v)), do: [v] def leaves(tree(l, r)), do: leaves(l) ++ leaves(r) end iex&gt; require BinTree iex&gt; import BinTree iex&gt; t = tree(leaf(1), ...&gt; tree(leaf(2), ...&gt; empty())) {:tree, {:leaf, 1}, {:tree, {:leaf, 2}, nil}} iex&gt; is_t(t) true iex&gt; leaves(t) [1, 2]","ref":"Tagged.html","title":"Tagged","type":"module"},{"doc":"Generates a macro that definies all things related to a tagged value tuple, {atom(), term()}. By default the macro has the same name as the tag, and all the things are generated.If tag is specified bare, as in deftagged ok, the constructor will have an arity of 1, and the type will wrap a term(), for backwards compatibility.deftagged ok &lt;=&gt; deftagged ok(term())If the tag is specified in the form of a parameterized type, the constructor will have the same arity as the specified type.When the constructor name is changed with as:, the type declaration belongs to the name, and not the tag.deftagged ok, as success(term())Keywordsas: name(...)Override default macro name. See Tagged.Constructor.type: falseOverride generation of type definition. See Tagged.Typedef.guard: falseOverride generation of guard expression macros. See Tagged.Guard.pipe_with: falseOverride generation of pipe filter. See Tagged.PipeWith.of: typedef DEPRECATED ~&gt; 0.4.0Declare the wrapped type statically, making it opaque. See Tagged.Typedef.","ref":"Tagged.html#deftagged/2","title":"Tagged.deftagged/2","type":"macro"},{"doc":"Generates macros for constructing, and destructuring tagged value tuples. By default, the macro name is the same as the tag, but can be overriden with the as: name keyword argument.This module is always executed.ExamplesDefine constructors with the same name as the tagdefmodule Tagged.Status use Tagged deftagged ok(term()) deftagged error(term()) end iex&gt; require Tagged.Status iex&gt; import Tagged.Status, only: [ok: 1] iex&gt; ok(:computer) {:ok, :computer} iex&gt; with ok(it) &lt;- Keyword.fetch([a: &quot;bacon&quot;], :a), do: &quot;Chunky \#{it}!&quot; &quot;Chunky bacon!&quot;Override constructor namedefmodule Tagged.Outcome use Tagged deftagged ok, as: success(term()) deftagged error, as: failure(term()) end iex&gt; require Tagged.Outcome iex&gt; import Tagged.Outcome, only: [failure: 1, success: 1] iex&gt; failure(:is_human) {:error, :is_human} iex&gt; with success(it) &lt;- {:ok, &quot;Computer&quot;}, do: &quot;OK, \#{it}!&quot; &quot;OK, Computer!&quot;","ref":"Tagged.Constructor.html","title":"Tagged.Constructor","type":"module"},{"doc":"Generates macros for use in guard expressions.Examplesiex&gt; require Outcome iex&gt; import Outcome iex&gt; f = fn x when is_success(x) -&gt; x; _ -&gt; success(nil) end iex&gt; success(:computer) |&gt; f.() {:ok, :computer} iex&gt; failure(:computer) |&gt; f.() {:ok, nil}","ref":"Tagged.Guard.html","title":"Tagged.Guard","type":"module"},{"doc":"Reasoning in terms of the outcome of an action.iex&gt; require Tagged.Outcome iex&gt; import Tagged.Outcome, only: [failure: 1, success: 1] iex&gt; failure(:is_human) {:error, :is_human} iex&gt; with success(it) &lt;- {:ok, &quot;Computer&quot;}, do: &quot;OK, \#{it}!&quot; &quot;OK, Computer!&quot;","ref":"Tagged.Outcome.html","title":"Tagged.Outcome","type":"module"},{"doc":"Constructor failure/1 for error tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; with failure(_) &lt;- {:error, 1}, do: true true iex&gt; with failure(_) &lt;- {:not_error, 1}, do: true {:not_error, 1}","ref":"Tagged.Outcome.html#failure/1","title":"Tagged.Outcome.failure/1","type":"macro"},{"doc":"Guard macro for testing if term is a error tagged tuple, with constructor failure/1.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; f = fn x when is_failure(x) -&gt; x; _ -&gt; nil end iex&gt; failure(1) |&gt; f.() {:error, 1} iex&gt; {:not_error, 1} |&gt; f.() nil","ref":"Tagged.Outcome.html#is_failure/1","title":"Tagged.Outcome.is_failure/1","type":"macro"},{"doc":"Guard macro for testing if term is a ok tagged tuple, with constructor success/1.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; f = fn x when is_success(x) -&gt; x; _ -&gt; nil end iex&gt; success(1) |&gt; f.() {:ok, 1} iex&gt; {:not_ok, 1} |&gt; f.() nil","ref":"Tagged.Outcome.html#is_success/1","title":"Tagged.Outcome.is_success/1","type":"macro"},{"doc":"Constructor success/1 for ok tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; with success(_) &lt;- {:ok, 1}, do: true true iex&gt; with success(_) &lt;- {:not_ok, 1}, do: true {:not_ok, 1}","ref":"Tagged.Outcome.html#success/1","title":"Tagged.Outcome.success/1","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a error tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; {:error, 1} ...&gt; |&gt; with_failure(fn _ -&gt; :match end) :match iex&gt; {:not_error, :miss} |&gt; with_failure(&amp; &amp;1) {:not_error, :miss}","ref":"Tagged.Outcome.html#with_failure/2","title":"Tagged.Outcome.with_failure/2","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a ok tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; {:ok, 1} ...&gt; |&gt; with_success(fn _ -&gt; :match end) :match iex&gt; {:not_ok, :miss} |&gt; with_success(&amp; &amp;1) {:not_ok, :miss}","ref":"Tagged.Outcome.html#with_success/2","title":"Tagged.Outcome.with_success/2","type":"macro"},{"doc":"","ref":"Tagged.Outcome.html#t:failure/0","title":"Tagged.Outcome.failure/0","type":"type"},{"doc":"","ref":"Tagged.Outcome.html#t:success/0","title":"Tagged.Outcome.success/0","type":"type"},{"doc":"Generates a function for selective execution, with pass-through of terms that does not match, much like the regular with ... &lt;- ..., do: ... construct.ExamplesGiven a module that defines a success() | failure() type:defmodule PipeWith do use Tagged deftagged not_an_integer(term()) deftagged not_a_number(term()) @type reason() :: not_an_integer() | not_a_number() deftagged success(integer()) deftagged failure(reason()) @type result() :: success() | failure() def validate_input(x) when is_integer(x), do: success(x) def validate_input(x), do: failure(not_an_integer(x)) def next_number(x), do: x + 1 def try_recovery({_, x}) when is_number(x), do: success(floor(x)) def try_recovery({_, x}), do: failure(not_a_number(x)) endThis is quite similar to the regular with ... &lt;- ..., do: ..., else: ... for happy paths:iex&gt; require PipeWith iex&gt; import PipeWith iex&gt; with success(v) &lt;- validate_input(1), ...&gt; do: next_number(v) 2 iex&gt; validate_input(1) ...&gt; |&gt; with_success(&amp;next_number/1) 2When the path is not a happy path, it offers more fluent control over recovery from failures at any point in the pipe:iex&gt; require PipeWith iex&gt; import PipeWith iex&gt; with success(v) &lt;- validate_input(0.7) do ...&gt; next_number(v) ...&gt; else ...&gt; failure(e) -&gt; with success(v) &lt;- try_recovery(e), ...&gt; do: next_number(v) ...&gt; end 1 iex&gt; validate_input(0.7) ...&gt; |&gt; with_failure(&amp;try_recovery/1) ...&gt; |&gt; with_success(&amp;next_number/1) 1","ref":"Tagged.PipeWith.html","title":"Tagged.PipeWith","type":"module"},{"doc":"","ref":"Tagged.PipeWith.html#gen_pipe_with/4","title":"Tagged.PipeWith.gen_pipe_with/4","type":"function"},{"doc":"Resoning in terms of the status of a result.iex&gt; require Tagged.Status iex&gt; import Tagged.Status, only: [ok: 1] iex&gt; ok(:computer) {:ok, :computer} iex&gt; with ok(it) &lt;- Keyword.fetch([a: &quot;bacon&quot;], :a), do: &quot;Chunky \#{it}!&quot; &quot;Chunky bacon!&quot;","ref":"Tagged.Status.html","title":"Tagged.Status","type":"module"},{"doc":"Constructor error/1 for error tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; with error(_) &lt;- {:error, 1}, do: true true iex&gt; with error(_) &lt;- {:not_error, 1}, do: true {:not_error, 1}","ref":"Tagged.Status.html#error/1","title":"Tagged.Status.error/1","type":"macro"},{"doc":"Guard macro for testing if term is a error tagged tuple, with constructor error/1.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; f = fn x when is_error(x) -&gt; x; _ -&gt; nil end iex&gt; error(1) |&gt; f.() {:error, 1} iex&gt; {:not_error, 1} |&gt; f.() nil","ref":"Tagged.Status.html#is_error/1","title":"Tagged.Status.is_error/1","type":"macro"},{"doc":"Guard macro for testing if term is a ok tagged tuple, with constructor ok/1.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; f = fn x when is_ok(x) -&gt; x; _ -&gt; nil end iex&gt; ok(1) |&gt; f.() {:ok, 1} iex&gt; {:not_ok, 1} |&gt; f.() nil","ref":"Tagged.Status.html#is_ok/1","title":"Tagged.Status.is_ok/1","type":"macro"},{"doc":"Constructor ok/1 for ok tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; with ok(_) &lt;- {:ok, 1}, do: true true iex&gt; with ok(_) &lt;- {:not_ok, 1}, do: true {:not_ok, 1}","ref":"Tagged.Status.html#ok/1","title":"Tagged.Status.ok/1","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a error tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; {:error, 1} ...&gt; |&gt; with_error(fn _ -&gt; :match end) :match iex&gt; {:not_error, :miss} |&gt; with_error(&amp; &amp;1) {:not_error, :miss}","ref":"Tagged.Status.html#with_error/2","title":"Tagged.Status.with_error/2","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a ok tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; {:ok, 1} ...&gt; |&gt; with_ok(fn _ -&gt; :match end) :match iex&gt; {:not_ok, :miss} |&gt; with_ok(&amp; &amp;1) {:not_ok, :miss}","ref":"Tagged.Status.html#with_ok/2","title":"Tagged.Status.with_ok/2","type":"macro"},{"doc":"","ref":"Tagged.Status.html#t:error/0","title":"Tagged.Status.error/0","type":"type"},{"doc":"","ref":"Tagged.Status.html#t:ok/0","title":"Tagged.Status.ok/0","type":"type"},{"doc":"Generates type definitions for tagged value tuples.This module is executed by default, but can be disabled for a whole module with keyword argument types: false to use/2, or for a single definition with keyword argument type: false to Tagged.deftagged/2.The wrapped type can be declared statically, which makes it opaque, by giving the of: keyword to Tagged.deftagged/2.ExamplesDisable type declaration for all tagged value tuple definitionsdefmodule DocTest.NoTypes do use Tagged, types: false deftagged foo end _iex&gt; use DocTest.NoTypes _iex&gt; t foo No type information for Kernel.foo was found or Kernel.foo is privateOverride type declaration for a single tagged value tuple definitiondefmodule DocTest.SomeTypes do use Tagged deftagged foo, type: false deftagged bar end _iex&gt; use DocTest.SomeTypes _iex&gt; t foo No type information for Kernel.foo was found or Kernel.foo is private _iex&gt; t bar @type bar() :: bar(term()) @type bar(t) :: {:bar, t} Tagged value tuple with a wrapped type t() \\\\ term()","ref":"Tagged.Typedef.html","title":"Tagged.Typedef","type":"module"},{"doc":"ChangelogAll notable changes to this project will be documented in this file.The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"FixedSome generated docs failed doctest for arity 0Some generated docs failed doctest for arity &gt; 1Generated docs failed doctest for nil tagGenerated type definition for arity 0 was {:tag} instead of :tag","ref":"changelog.html#v0-4-1-2020-05-31","title":"Changelog - v0.4.1 - 2020-05-31","type":"extras"},{"doc":"AddedSupport for different arities, forming Sum Arithmetic Data Types.ChangedTotal rework of how to specify wrapped types.All types are fully statically defined.Reverted the opaque type generation.Deprecate the use of the examples Tagged.Status and Tagged.Outcome.FixedThe type requirement on the of: keyword was too strict, breaking nontrivial type specifications.","ref":"changelog.html#v0-4-0-2020-05-27","title":"Changelog - v0.4.0 - 2020-05-27","type":"extras"},{"doc":"AddedGenerate defguard macros.Option to declare the wrapped type statically.ChangedGenerated typedefs are opaque.","ref":"changelog.html#v0-3-0-2020-05-24","title":"Changelog - v0.3.0 - 2020-05-24","type":"extras"},{"doc":"AddedSelective function execution in pipes.ChangedOption keywords are now validated on both module and macro level.FixedModule level use options was not properly unwrapped.Generated constructor doc examples referenced the wrong module.Code formatting was not configured to keep deftagged without parenthesis.","ref":"changelog.html#v0-2-0-2020-05-22","title":"Changelog - v0.2.0 - 2020-05-22","type":"extras"},{"doc":"Initial package release.AddedDefine deftagged macro, used to declare a tagged value tuple.Generate a constructor / destructuring macro for a tag.Generate a typedef for a tag.","ref":"changelog.html#v0-1-0-2020-05-19","title":"Changelog - v0.1.0 - 2020-05-19","type":"extras"}]